# Quickly setup a project skeleton

import os
import shutil
import subprocess

from .api import secret

project = input("Enter project name (empty to quit): ")
if project:
    print("This will create a project skeleton using Tokit in current directory")
    source = os.path.join(os.path.dirname(__file__), 'skeleton')
    destination = os.path.join(os.getcwd(), project)
    print("Destination: ", destination)
    shutil.copytree(source, destination)

    commands = [
        "cd %s" % destination,
        """LC_ALL=C find config -name '*.*' -type f \
            -exec sed -i '' s/PR0JECT/%s/g {} \; """ % project,

        'cp config/development.sample.ini config/development.ini',
        'cp config/production.sample.ini config/production.ini',
        'echo "\n[secret]\ncookie_secret=%s\n" >> config/production.ini' % secret(),

        "pyvenv . ",
        "source bin/activate",
        "pip3 install -r src/requirements.txt",
        "python3 src/app.py"
    ]

    subprocess.call('; '.join(commands), stderr=subprocess.STDOUT, shell=True)



import tokit

from tokit.api import ErrorMixin, JsonMixin

class Config(tokit.Config):
    timezone = 'Asia/Tokyo'

def _addition(config):
    config.settings.update({
        'static_path': 'static'
    })

tokit.Event.get('config').attach(_addition)


class Websocket(ErrorMixin, tokit.Websocket):
    _repo_ = 'Request'


class Request(ErrorMixin, tokit.Request):
    _repo_ = 'Request'

    def css(self):
        return ['style.css']

    def js(self):
        return ['vendor/riot+compiler.js', 'global.js']



import logging
from tornado.gen import coroutine
import momoko

logger = logging.getLogger(__name__)

import tokit


def pg_init(app):
    """ Hook to init Postgres momoko driver.
    dsn config is required, with syntax same as Psycopg2 DSN.
    
    Sample env.ini::
    
        [postgres]
        dsn=dbname=[APP_NAME]
        size=2
    """
    postgres = app.config.env['postgres']
    app.db = momoko.Pool(dsn=postgres.get('dsn'), size=postgres.getint('size'))
    app.db.connect()


def pg_debug(_):
    for lg in [logger, logging.getLogger('momoko')]:
        lg.setLevel(logging.DEBUG)


tokit.Event.get('debug').attach(pg_debug)
tokit.Event.get('init').attach(pg_init)


class PgMixin:
    @property
    def db(self):
        """
        :return: momoko.Pool
        """
        return self.application.db

    @coroutine
    def pg_insert(self, table, fields=None, **data):
        """
        Postgres shorcut to insert data
        
        Example::

            user_id = yield self.pg_insert('users', {"username": "foo", "password": "secret"})

        .. warning::
            Must decorate caller method with ``tornado.gen.coroutine`` because this is async

        :return int row's id
        """
        if fields:
            data = list(self.get_request_dict(*fields).values())
        else:
            fields = list(data.keys())
            values = list(data.values())
        assert len(data) > 0  # check data

        sql = 'INSERT INTO {} ({}) VALUES ({}) RETURNING id ' \
            .format(table,
                    ','.join(fields),
                    ','.join(['%s'] * len(fields))
                    )
        cursor = yield self.db.execute(sql, values)
        return cursor.fetchone()[0]

    @coroutine
    def pg_update(self, table, row_id, data):
        """
        Postgres shorcut to update data.

        .. warning::
            Must decorate callee with ``tornado.gen.coroutine`` because this is async

        """
        changes = [field + ' = %s' for field in data.keys()]
        sql = 'UPDATE {} SET {} WHERE id = %s'.format(table, ','.join(changes))
        values = list(data.values())
        values.append(row_id)
        yield self.db.execute(sql, values)

    # Aliases
    db_insert = pg_insert
    db_update = pg_update



#!/usr/bin/env python
from tofab import *

env.x.app = 'PR0JECT'
env.hosts = ['PR0JECT.com']
env.x.base_port = 7380
env.x.n_instances = 2
env.x.remote_path = '/home/PR0JECT'
env.x.wait = 3 # (second) time wait for loading complete
env.user = env.x.app
env.port = 9622



#!/usr/env python3
import tokit
import os
import logging

import _app

from tornado.options import define, options, parse_command_line
define('port', default='7380')
define('env', default=None)
parse_command_line()

def get_config(env_name=None):
    config = _app.Config(__file__)
    os.chdir(config.root_path)
    config.env_name = options.env or env_name or os.environ.get('ENV', 'development')
    try:
        # Add common.ini if needed to share
        config.load(['../config/%s.ini' % config.env_name])
    except FileNotFoundError as e:
        logging.warning('Config file is not present')
    logging.info('Env: ' + config.env_name)    
    return config

if __name__ == '__main__':
    tokit.start(options.port, get_config())



import logging
import os

from cassandra.cluster import Cluster
from cassandra.cqlengine import connection as cqlengine_connection
from tornado.gen import coroutine
from tornado.concurrent import Future
from tornado.ioloop import IOLoop

import tokit
from sqlbuilder.smartsql import Table, Query, compile

logger = tokit.logger

def allow_management():
    os.environ['CQLENG_ALLOW_SCHEMA_MANAGEMENT'] = 'CQLENG_ALLOW_SCHEMA_MANAGEMENT'
    
def cassandra_init(app):
    """
    Sample configuration::

        [cassandra]
        contact_points=
            192.168.1.1
            192.168.1.2
        port=9042
        keyspace=blabla
    """
    try:
        config = app.config.env['cassandra']
    except KeyError:
        logger.warn('Cassandra was not configured')
        return
    hosts = [p.strip() for p in config.get('contact_points').split('\n')]
    # For use with controller
    app.cassandra_cluster = Cluster(
        contact_points=hosts,
        port=int(config.get('port')),
        connect_timeout=1,
    )
    # For use with object mapper
    cqlengine_connection.setup(
        hosts,
        default_keyspace=config.get('keyspace'),
        lazy_connect=True, retry_connect=True
    )
    # Further hook
    tokit.Event.get('cassandra_init').emit(app)

tokit.Event.get('init').attach(cassandra_init)


class CassandraMixin():
    """
    Reference: http://datastax.github.io/python-driver/api/cassandra/cluster.html#cassandra.cluster.Session.execute_async
    """

    @property
    def db(self, keyspace=None):
        """
        Get Cassandra session
        """
        if not keyspace:
            keyspace = self.application.config.env['cassandra'].get('keyspace')
        return self.application.cassandra_cluster.connect(keyspace)

    @coroutine
    def cs_insert(self, table, data):
        t = getattr(Table, table)
        statement = Query(t).insert(data)
        return self.cs_query(statement)

    @coroutine
    def cs_update(self, table, row_id, data):
        t = getattr(Table, table)
        statement = Query(t).where(t.id == row_id).update(data)
        return self.cs_query(statement)

    def cs_query(self, statement):
        """
        statement - a tuple (sql, params) or a `sqlbuilder.Query` instance
        Reference: http://alexapps.net/cassandra-asynchronous-future-calls-pyth/
        """
        query = statement
        if isinstance(query, Query):
            query = compile(statement)
        future = Future()
        logger.debug('Cassandra executes: %s', query)
        # Cassanra driver use a different thread
        cs_future = self.db.execute_async(*query)
        
        def _success(result):
            IOLoop.instance().add_callback(future.set_result, result)

        def _fail(e):
            IOLoop.instance().add_callback(future.set_exception, e)

        cs_future.add_callbacks(_success, _fail)
        return future

    # Aliases
    db_insert = cs_insert
    db_update = cs_update
    db_query = cs_query



#!/usr/bin/env python3
"""
@manhgd/tokit
A kit for development with Tornado web framework.
"""

__all__ = ['MetaRepo', 'Repo', 'Request', 'Websocket', 'Module', 'Static', 'Config']

import os, sys, subprocess
import re
import collections
import logging
import time
import signal
import json
import importlib
import inspect
import configparser
from contextlib import contextmanager

import tornado.locale
import tornado.httpserver
import tornado.web
import tornado.websocket
import tornado.netutil
from tornado.gen import coroutine
from tornado.ioloop import IOLoop

logger = logging.getLogger('tokit')


def to_json(obj):
    return json.dumps(obj, ensure_ascii=False).replace("</", "<\\/")


class Repo:
    """ Decorator-based registry of objects

    Example::

    """

    _repo = collections.defaultdict(list)

    def __init__(self, name=None):
        self.name = name

    def __call__(self, obj):
        if not self.name:
            if inspect.isclass(obj):
                parent_cls, *_ = inspect.getmro(obj)
                self.name = parent_cls.__name__
        self._repo[self.name].append(obj)
        return obj

    @classmethod
    def known(cls, name):
        """ Get registered objects
        :return: list
        """
        return cls._repo.get(name, [])


class MetaRepo(type):
    """ A metaclass to make subclasses registry for any class """
    _repo = collections.defaultdict(list)

    def __init__(cls, name, bases, nmspc, **kwarg):
        # Instance object
        super(MetaRepo, cls).__init__(name, bases, nmspc)
        # Register
        repo_name = getattr(cls, '_repo_', None)
        if not repo_name:
            *_, base_cls = bases
            repo_name = base_cls.__name__
        MetaRepo._repo[repo_name].append(cls)

    @classmethod
    def known(mcs, parent_name) -> list:
        """ Get registered subclass of ``parent_name``
        :param str parent_name: pure class name without module prefix
        """
        return MetaRepo._repo[parent_name]


class Request(tornado.web.RequestHandler, metaclass=MetaRepo):
    """
    Base class for handling request
    Class hierarchy is defined right to left, methods are resolved is from left to right
    http://www.ianlewis.org/en/mixins-and-python
    Therefore, mixin if exists, should be added to the left of declare::

        class Foo(BarMixin, Request):
            pass
    """

    _route_ = None
    """
    Route can be pattern or (pattern, name) or an URLSpec::

        class Post(Request):
            _route_ = r'/post/.*'
            def get(self, slug):
                pass # Add logic here
    """

    def set_default_headers(self):
        self.set_header('Server', 'Python3')

    def abs_url(self, *args):
        return self.request.protocol + "://" + self.request.host + self.reverse_url(*args)

    def get_template_namespace(self):
        namespace = super(Request, self).get_template_namespace()
        namespace['json'] = to_json
        namespace['url'] = self.abs_url
        return namespace

    def js(self):
        """ List (to preserved ordering) of JS path to to used by layout file """
        return []

    def css(self):
        """ List (to preserved ordering) of CSS path to to used by layout file """
        return []

    def get_request_dict(self, *args):
        """ Return dict of request arguments """
        return collections.OrderedDict(
            (field, self.get_body_argument(field, None)) for field in args)

    def redirect_referer(self):
        return self.redirect(self.request.headers.get('Referer', '/'))

    @property
    def env(self):
        return self.application.config.env

    @classmethod
    def known(cls):
        """ Get Request's subclasses
        TODO add weight or ordering important routes
        """
        routes = []
        for handler in MetaRepo.known(cls.__name__):
            route = getattr(handler, '_route_', None)
            if not route:
                logger.debug("Missing route for handler %s.%s",
                             handler.__module__, handler.__name__)
                continue
            if isinstance(route, str):
                routes.append(tornado.web.URLSpec(route, handler))
            elif isinstance(route, tornado.web.URLSpec):
                routes.append(route)
            else:
                pattern, name = route
                routes.append(tornado.web.URLSpec(pattern, handler, name=name))
        return routes


class Websocket(tornado.websocket.WebSocketHandler, metaclass=MetaRepo):
    def reply(self, _payload=None, **kwargs):
        self.write_message(_payload or to_json(kwargs))

    @property
    def env(self):
        return self.application.config.env


class Module(tornado.web.UIModule, metaclass=MetaRepo):
    """ Subclass this to create a UIModule
    It's available as class name::

        class Table(Module):
            def render(self):
                pass # then in template, you can use {% module Table() %}

    """

    @classmethod
    def known(cls):
        return {c.__name__: c for c in MetaRepo.known(cls.__name__)}


class Static(tornado.web.StaticFileHandler):
    ALLOW_TYPES = 'tag', 'js', 'css', 'png', 'jpg', 'svg',  'gif', 'zip', 'tar', 'tgz', 'txt'
    VALID_PATH = re.compile(r'.*\.({types})$'.format(types='|'.join(ALLOW_TYPES)))

    def validate_absolute_path(self, root, absolute_path):
        """ Add file types checking """
        if not self.VALID_PATH.match(absolute_path):
            raise tornado.web.HTTPError(403, 'Unallowed file type')
        return absolute_path

class Event:
    """Event handlers storage.

    Example:

    >>> def handler(**kwargs):
    ...     print("Triggered:", kwargs)
    ...
    >>> Event.get('init').attach(handler)
    >>> Event.get('init').emit(status='OK')
    Triggered: {'status': 'OK'}

    """

    _repo = {}

    def __init__(self, name):
        self._handlers = set()
        self.name = name

    def attach(self, task):
        self._handlers.add(task)

    def detach(self, task):
        self._handlers.remove(task)

    @classmethod
    def get(cls, name):
        instance = cls._repo.get(name, None)
        if not instance:
            instance = cls(name)
            cls._repo[name] = instance
        return instance

    @contextmanager
    def subscribe(self, *tasks):
        for task in tasks:
            self.attach(task)
        try:
            yield
        finally:
            for task in tasks:
                self.detach(task)

    def emit(self, *args, **kwargs):
        for handler in self._handlers:
            handler(*args, **kwargs)


class AttributeDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__


class Config:
    """ Subclass this to customize runtime config """

    settings = dict(
        compress_response=True,
        static_path='.',
        static_url_prefix='/static/',
        static_handler_class=Static,
        debug=True,
        compiled_template_cache=False,
        cookie_secret='TODO'
    )
    x = AttributeDict()
    root_path = None
    in_production = False
    timezone = 'UTC'
    locale = 'en'

    modules = None
    """ List of dirname, as a module """

    kill_blocking = 2
    """ (second) time to kill process if blocking too long """

    session_timeout = 60 * 24 * 3600
    env_name = None
    env = {}

    def __init__(self, base_file):
        self.root_path = os.path.abspath(os.path.dirname(base_file))
        self.settings['static_path'] = os.path.join(self.root_path, self.settings['static_path'])
        lang_path = os.path.join(self.root_path, 'lang')
        if os.path.exists(lang_path):
            tornado.locale.load_translations(lang_path)

    def load(self, cfg_files=None):
        """ Load extra env config
        :param: list cfg_files relative path to project root
        """
        self.env = configparser.ConfigParser()
        main, *overrides = [os.path.join(self.root_path, f) for f in cfg_files]
        self.env.read_file(open(main))
        if len(overrides):
            self.env.read(overrides)

        Event.get('env').emit(self.env)
        self.setup()
        Event.get('config').emit(self)

    def setup(self):
        self.in_production = self.env['app'].getboolean('in_production')
        self.settings['debug'] = self.env['app'].getboolean('debug')
        log_level = self.env['app'].get('log_level')
        logging.basicConfig(level=getattr(logging, log_level))
        self.settings['cookie_secret'] = self.env['secret'].get('cookie_secret')
        dns_resolver = self.env['app'].get('dns_resolver', 'tornado.netutil.ThreadedResolver')
        tornado.netutil.Resolver.configure(dns_resolver)


class App(tornado.web.Application):
    config = None

    @classmethod
    def instance(cls, config):
        load(config)
        Event.get('config').emit(config)
        config.settings['ui_modules'] = Module.known()
        app = App(**config.settings)
        app.config = config
        Event.get('init').emit(app)
        Event.get(config.env_name).emit(app)
        app.add_handlers('.*$', Request.known())
        return app


def load(config):
    """ Import declared modules from config, during this imports, routes
    and other components will be registered """
    os.environ['TZ'] = config.timezone
    time.tzset()
    tornado.locale.set_default_locale(config.locale)
    if not config.modules:
        # Search all modules as a dir,
        _, config.modules, _ = next(os.walk(config.root_path))
        # Exclude special dirnames
        config.modules = [m for m in config.modules \
                          if not (m.startswith('.') or m.startswith('_'))]
    loaded = []
    for m in config.modules:
        try:
            importlib.import_module(m)
            loaded.append(m)
        except TypeError:
            pass
        except SyntaxError:
            ex = sys.exc_info()
            logger.error("Broken module: %s %s %s", ex[0].__name__,
                         os.path.basename(
                             sys.exc_info()[2].tb_frame.f_code.co_filename),
                         ex[2].tb_lineno)
            sys.exit(1)
    logger.info('Autoloaded modules: %s', loaded)


def start(port, config):
    """ Entry point for application. This setups IOLoop, load classes and run HTTP server """
    app = App.instance(config)

    http_server = tornado.httpserver.HTTPServer(app, xheaders=True)
    ioloop = IOLoop.instance()

    def _graceful():
        def _shutdown():
            pass

        ioloop.stop()
        logger.info('Shutting down...')
        http_server.stop()
        ioloop.call_later(1, _shutdown)

    def _on_term(*args):
        ioloop.add_callback_from_signal(_graceful)

    http_server.listen(port, 'localhost')
    logger.info('Running PID {pid} @ localhost:{port}'.format(pid=os.getpid(), port=port))
    if config.in_production:
        # Automatically kill if anything blocks the process
        signal.signal(signal.SIGTERM, _on_term)
        ioloop.set_blocking_log_threshold(1)
        ioloop.set_blocking_signal_threshold(config.kill_blocking, action=None)
    try:
        Event.get('start').emit(app)
        ioloop.start()
    except KeyboardInterrupt:
        logger.info('Bye.')



import json
import traceback
import sys
import functools
import string
import random

import tokit

from tornado.websocket import WebSocketHandler


def secret(length=16):
    return ''.join(random.SystemRandom(). \
            choice(string.ascii_uppercase + string.digits) \
            for _ in range(length))

def parse_json(s):
    try:
        return json.loads(s)
    except ValueError as e:
        raise Exception('Invalid JSON: ' + str(e))


def api_auth(method):
    """
    Require auth cookie to access an API
    """

    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        if not self.current_user:
            self.write_exception(Exception('Login required'))
        return method(self, *args, **kwargs)

    return wrapper


class JsonMixin:
    """ Auto parse JSON request body and store in self.data """

    def prepare(self):
        super().prepare()
        self.data = None
        if self.request.body:
            try:
                self.data = parse_json(self.request.body.decode())
            except ValueError as e:
                self.write_exception(e)


class ErrorMixin:
    """ Show errors in JSON instead of default HTML

    Always check key "status" from result, will be "ok" or "error"
    """

    def write_exception(self, e):
        error_type = e.__class__.__name__
        if self.settings['debug']:
            self.write_error(error_type, exc_info=sys.exc_info())
        else:
            self.write_error(error_type, reason=str(e))

    def write_error(self, status_code, reason=None, **kwargs):
        response = {
            'status': 'error',
            'code': status_code,
        }
        if self.settings.get("serve_traceback") and "exc_info" in kwargs:
            response['detail'] = traceback.format_exception(*kwargs["exc_info"])
        else:
            response['reason'] = reason
        if isinstance(self, WebSocketHandler):
            self.write_message(response)
        else:
            self.write(response)
            self.finish()



from distutils.core import setup

# Thanks to http://peterdowns.com/posts/first-time-with-pypi.html
tokit_version = '0.4.0'
setup(
    name='tokit',
    packages=['tokit'],
    version=tokit_version,
    description=r'''A kit for development with Tornado web framework.
        'See https://github.com/manhg/writekit for usage demo.''',
    author='Giang Manh',
    author_email='manhgd@yahoo.com',
    url='https://github.com/manhg/tokit',
    download_url='https://github.com/manhg/tokit/tarball/' + tokit_version,
    keywords=['tornado', 'web', 'tokit'],
    classifiers=[
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    install_requires=[
        'tornado',
        # 'cassandra-driver',
        # 'momoko',
    ],
    dependency_links=[
        # 'https://github.com/tornadoweb/tornado/archive/v4.2.1.tar.gz',
        # 'https://github.com/FSX/momoko/archive/v2.2.0.tar.gz',
        # 'https://github.com/datastax/python-driver/archive/2.7.2.tar.gz',
    ],
)



import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../src")))

master_doc = "index"
project = "pokersu"


extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.extlinks",
    "sphinx.ext.intersphinx",
    ]

primary_domain = 'py'
default_role = 'py:obj'
autodoc_member_order = "bysource"
autodoc_default_flags = ['members', 'undoc-members', 'private-members']
autoclass_content = "both"

# Without this line sphinx includes a copy of object.__init__'s docstring
# on any class that doesn't define __init__.
# https://bitbucket.org/birkenfeld/sphinx/issue/1337/autoclass_content-both-uses-object__init__
autodoc_docstring_signature = False
coverage_skip_undoc_in_source = True
coverage_ignore_classes = []
coverage_ignore_functions = []


intersphinx_mapping = {'python': ('http://docs.python.org/', None)}

html_theme = "sphinx_rtd_theme"
